//
//  ViewController.swift
//  CrashTest
//
//  Created by Oleksandr Kirichenko on 2/15/17.
//
//

import UIKit

class ViewController: UIViewController {
    @IBOutlet weak var toBottomConstraint: NSLayoutConstraint!
    let formatter: DateFormatter = {
        let formatter = DateFormatter()
        formatter.dateFormat = "yyyy MM dd"
        formatter.timeZone = Calendar.current.timeZone
        formatter.locale = Calendar.current.locale
        return formatter
    }()
    
    var weekMode = false {
        didSet {
            if weekMode {
                toBottomConstraint.constant = UIScreen.main.bounds.height * 0.8
                
                calendarView.scrollDirection = .horizontal
                calendarView.itemSize = nil
            } else {
                toBottomConstraint.constant = 0
                
                calendarView.scrollDirection = .vertical
                calendarView.itemSize = floor(UIScreen.main.bounds.height / 12.5)
            }
        }
    }
    
    @IBOutlet weak var calendarView: JTAppleCalendarView! {
        didSet {
            calendarView.delegate = self
            calendarView.dataSource = self
            calendarView.registerCellViewXib(file: "CellView")
        }
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        
        weekMode = false
    }
    
}

extension ViewController: JTAppleCalendarViewDataSource {
    func configureCalendar(_ calendar: JTAppleCalendarView) -> ConfigurationParameters {
        let startDate = formatter.date(from: "2017 01 01")!
        let endDate = formatter.date(from: "2018 01 01")!
        
        let parameters = ConfigurationParameters(startDate: startDate,
                                                 endDate: endDate,
                                                 numberOfRows: weekMode ? 1 : 6,
                                                 generateInDates: weekMode ? .forFirstMonthOnly : .forAllMonths,
                                                 generateOutDates: weekMode ? .off : .off,
                                                 firstDayOfWeek: .monday,
                                                 hasStrictBoundaries: false )
        
        return parameters
    }
}

extension ViewController: JTAppleCalendarViewDelegate {
    func calendar(_ calendar: JTAppleCalendarView, didSelectDate date: Date, cell: JTAppleDayCellView?, cellState: CellState) {
        weekMode = !weekMode
        
        // I have now update the source code.
        // Reload should NOT happen automatically when you change the frame. The developer should do this for himself if needed.
        
        calendarView.reloadData {
            self.calendarView.scrollToDate(date, triggerScrollToDateDelegate: false, animateScroll: false)
        }
    }
    
    func calendar(_ calendar: JTAppleCalendarView, willDisplayCell cell: JTAppleDayCellView, date: Date, cellState: CellState) {
        guard let myCustomCell = cell as? CellView else { return }
        
        myCustomCell.dayLabel.text = cellState.text
    }
}
